
/**
 * This Java class can find the location of the sun without requiring time-
 * consuming functions from the math library and is accurate to within half a
 * degree.
 * 
 * Source: http://susdesign.com/sunangle
 *
 * @author d
 */
public class SunLocator
{
   private double mLat = 35.3039605; //Latitude for the mech lab
   private double mLat2 = 32.386677; //Latitude for NW side of Tucson
   private double mLong = -120.6635398; //Longitude for the mech lab
   private double mLong2 = -111.043926; //Longitude for NW side of Tucson
   private double mGMTOffset = -8; //Pacific time
   private double mGMTOffset2 = -7; //Arizona time
   private double mElev = 308; //the elevation in feet
   private double mElev2 = 2487; //the elevation for northwest Tucson

	private double mMagDecl = 13.0 + 26.0/60.0; //13 deg. 26' E
	private double mMagDecl2 = 10.0 + 37.0/60.0; //10 deg. 37' E for Tucson

   //default test time will be 1620 hours
   private double getInputHr()
   {
      return 15;
   }

   private float getInputHrf()
   {
      return 15.0f;
   }

   private double getInputMin()
   {
      return 15.0 * 60.0d;
   }

   private float getInputMinf()
   {
		return 15.0f * 60.0f;
   }

   private int getInputYear()
   {
      return 2010;
   }

   private int getInputMonth()
   {
      return 3;
   }

   private int getInputDay()
   {
      return 9;
   }

   private int getInputDate()
   {
      return getInputDay();
      //return 1265732400;
   }

   private int getCorrMonth()
   {
      int monthNo = getInputMonth();
      return (monthNo > 2) ? monthNo - 3 : monthNo + 9;
   }

   private int getCorrYear()
   {
      int yearNo = getInputYear();
      int monthNo = getInputMonth();
      return (monthNo > 2) ? yearNo : yearNo - 1;
   }

   private double getT(double univTime)
   {
      double univDays = univTime / 24.0d;
      int inputDate = getInputDate();
      int corrMonth = getCorrMonth();
      int corrYear = getCorrYear();
      double valMonth = Math.floor(30.6 * corrMonth + 0.5);
      double valYear = Math.floor(365.25 * (corrYear - 1976));
      double valT = univDays + inputDate + valMonth + valYear - 8707.5;
      return valT / 36525.0;
   }

   private float getT(float univTime)
   {
      float univDays = univTime / 24.0f;
      int inputDate = getInputDate();
      int corrMonth = getCorrMonth();
      int corrYear = getCorrYear();
      float valMonth = (float) Math.floor(30.6f * corrMonth + 0.5f);
      float valYear = (float) Math.floor(365.25f * (corrYear - 1976));
      float valT = univDays + inputDate + valMonth + valYear - 8707.5f;
      return valT / 36525.0f;
   }

   private double getC(double gVal)
   {
      double gRads = Math.toRadians(gVal);
      double valC1 = 1.915 * Math.sin(gRads);
      double valC2 = 0.020 * Math.sin(2.0 * gRads);
      return valC1 + valC2;
   }

   private float getC(float gVal)
   {
      float gRads = (float) Math.toRadians(gVal);
      float valC1 = 1.915f * FastMath.sine2(gRads);
      float valC2 = 0.020f * FastMath.sine2(2.0f * gRads);
      return valC1 + valC2;
   }

   private double getAlpha(double lVal)
   {
      double lRads = Math.toRadians(lVal);
      double valAlpha1 = 2.466 * Math.sin(2.0 * lRads);
      double valAlpha2 = 0.053 * Math.sin(4.0 * lRads);
      return lVal - valAlpha1 + valAlpha2;
   }

   private float getAlpha(float lVal)
   {
      float lRads = (float) Math.toRadians(lVal);
      float valAlpha1 = 2.466f * FastMath.sine2(2.0f * lRads);
      float valAlpha2 = 0.053f * FastMath.sine2(4.0f * lRads);
      return lVal - valAlpha1 + valAlpha2;
   }

   private double getDeclAngle(double obliqAngle, double alpha)
   {
      double obliqRad = Math.toRadians(obliqAngle);
      double alphaRad = Math.toRadians(alpha);
      double declRad = Math.atan(Math.tan(obliqRad) * Math.sin(alphaRad));
      return Math.toDegrees(declRad);
   }

   private float getDeclAngle(float obliqAngle, float alpha)
   {
      float obliqRad = (float) Math.toRadians(obliqAngle);
      float alphaRad = (float) Math.toRadians(alpha);
      float declRad = FastMath.arctan(FastMath.tangent2(obliqRad) *
       FastMath.sine2(alphaRad));
      return (float) Math.toDegrees(declRad);
   }

   private double getAltAngle(double declAngle, double latitude,
    double solarAngle)
   {
      double declRad = Math.toRadians(declAngle);
      double solarRad = Math.toRadians(solarAngle + 180);
      double latRad = Math.toRadians(mLat);
      double valAlt1 = Math.sin(latRad) * Math.sin(declRad);
      double valAlt2 = Math.cos(latRad) * Math.cos(declRad)
       * Math.cos(solarRad);
      return Math.toDegrees(Math.asin(valAlt1 - valAlt2));
   }

   private float getAltAngle(float declAngle, float latitude, float solarAngle)
   {
      float declRad = (float) Math.toRadians(declAngle);
      float solarRad = (float) Math.toRadians(solarAngle + 180);
      float latRad = (float) Math.toRadians(mLat);
      float valAlt1 = FastMath.sine2(latRad) * FastMath.sine2(declRad);
      float valAlt2 = FastMath.cosine2(latRad) * FastMath.cosine2(declRad)
       * FastMath.cosine2(solarRad);
      return (float) Math.toDegrees(FastMath.arcsin(valAlt1 - valAlt2));
   }

   private double getPreAzimuth(double declAngle, double latitude,
    double solarHourAngle, double altAngle)
   {
      double declRad = Math.toRadians(declAngle);
      double solarRad = Math.toRadians(solarHourAngle + 180);
      double altRad = Math.toRadians(altAngle);
      double latRad = Math.toRadians(latitude);
      double valAzi1 = Math.cos(latRad) * Math.tan(declRad);
      double valAzi2 = Math.sin(latRad) * Math.cos(solarRad);
      double valAzi3 = Math.cos(declRad) * (valAzi1 + valAzi2);
      double aziRad = Math.acos(valAzi3 / Math.cos(altRad));
      double preAzimuth = Math.toDegrees(aziRad);
      return preAzimuth;
   }

   private float getPreAzimuth(float declAngle, float latitude,
    float solarHourAngle, float altAngle)
   {
      float declRad = (float) Math.toRadians(declAngle);
      float solarRad = (float) Math.toRadians(solarHourAngle + 180);
      float altRad = (float) Math.toRadians(altAngle);
      float latRad = (float) Math.toRadians(latitude);
      float valAzi1 = FastMath.cosine2(latRad) * FastMath.tangent2(declRad);
      float valAzi2 = FastMath.sine2(latRad) * FastMath.cosine2(solarRad);
      float valAzi3 = FastMath.cosine2(declRad) * (valAzi1 + valAzi2);
      float aziRad = FastMath.arccos(valAzi3 / FastMath.cosine2(altRad));
      float preAzimuth = (float) Math.toDegrees(aziRad);
      return preAzimuth;
   }

   private double getDaylightMin(double declAngle, double latitude, double elev)
   {
      double declRad = Math.toRadians(declAngle);
      double latRad = Math.toRadians(latitude);
      double elevInM = elev * 12 * 2.54 / 100;
      double elevRad = Math.toRadians(-0.8333 - 0.0347 * Math.sqrt(elevInM));
      double riseVal1 = Math.sin(elevRad);
      double riseVal2 = Math.sin(latRad) * Math.sin(declRad);
      double riseVal3 = Math.cos(latRad) * Math.cos(declRad);
      double riseRad = Math.acos((riseVal1 - riseVal2) / riseVal3);
      return Math.toDegrees(riseRad) * 4;
   }

   private float getDaylightMin(float declAngle, float latitude, float elev)
   {
      float declRad = FastMath.toRadians(declAngle);
      float latRad =  FastMath.toRadians(latitude);
      float elevInM = elev * 12.0f * 2.54f / 100.0f;
      float elevRad = FastMath.toRadians(-0.8333f - 0.0347f * FastMath.sqrt(elevInM));
      float riseVal1 = FastMath.sine2(elevRad);
      float riseVal2 = FastMath.sine2(latRad) * FastMath.sine2(declRad);
      float riseVal3 = FastMath.cosine2(latRad) * FastMath.cosine2(declRad);
      float riseRad = FastMath.arccos((riseVal1 - riseVal2) / riseVal3);
      return FastMath.toDegrees(riseRad) * 4.0f;
   }

   private double norm360(double num)
   {
      return num - Math.floor(num / 360.0d) * 360;
   }

   private float norm360(float num)
   {
      return num - (float) Math.floor(num / 360.0f) * 360.0f;
   }

   private double norm180(double num)
   {
      double newNum = norm360(num);
      return (newNum > 180) ? newNum - 360 : newNum;
   }

   private float norm180(float num)
   {
      float newNum = norm360(num);
      return (newNum > 180.0f) ? newNum - 360.0f : newNum;
   }

   private double norm24(double num)
   {
      return num - Math.floor(num / 24.0) * 24;
   }

   private float norm24(float num)
   {
      return num - (float) Math.floor(num / 24.0f) * 24.0f;
   }

   private String degSign()
   {
      return "\u00B0";
   }

   private int[] getHourMin(double totalMin)
   {
      int hourMin[] = new int[2];
      hourMin[0] = (int) Math.floor(totalMin / 60);
      hourMin[1] = (int) Math.floor(totalMin - 60.0 * hourMin[0]);
      return hourMin;
   }

   private int[] getHourMin(float totalMin)
   {
      int hourMin[] = new int[2];
      hourMin[0] = (int) Math.floor(totalMin / 60);
      hourMin[1] = (int) Math.floor(totalMin - 60.0 * hourMin[0]);
      return hourMin;
   }

   private void run()
   {
      double longitude = mLong;
      double latitude = mLat;
      double elev = mElev;
      double offset = mGMTOffset;

      double utslo = getInputHr() - offset;
      double tVal = getT(utslo);
      double gVal = norm360(357.528 + 35999.05 * tVal);

      double cVal = getC(gVal);
      double lVal = norm360(280.46 + (tVal * 36000.77) + cVal);

      double alpha = getAlpha(lVal);
      double ghaVal = norm360(utslo * 15 - 180 - cVal + lVal - alpha);

      double obliqAngle = 23.4393 - 0.013 * tVal;

      double declAngle = getDeclAngle(obliqAngle, alpha);
      System.out.printf("Declination: %.3f%s\n", declAngle, degSign());

      double deltaEOT = (lVal - cVal - alpha) / 15.0; //EOT (hours)
      double totalSecs = deltaEOT * 3600; //EOT (total seconds)
      int minEOT = (int) Math.floor(totalSecs / 60.0); //EOT (min)
      int secEOT = (int) Math.round(totalSecs - 60 * minEOT); //EOT (sec)

      System.out.printf("Equation of time (min): %d:%02d\n", minEOT, secEOT);

      //DST gets added to this if active
      double deltaLSOT = (-longitude + (15 * offset)) / 15 - deltaEOT;

      double solarHourAngle = norm180(ghaVal + longitude); //GHA - signed long
      double solarTime = norm24(12 + solarHourAngle / 15.0);

      double solarMin = getInputMin() - deltaLSOT * 60.0;
      int solarDay = (solarMin < 0) ? -1 : (solarMin >= 1440) ? 1 : 0;
      solarMin -= 1440 * solarDay;
      int solarH = (int) Math.floor(solarMin / 60);
      int solarM = (int) Math.floor(solarMin - 60.0d * solarH);
      String dayStr = (solarDay == -1) ? "-" : (solarDay == 1) ? "+" : "";
      System.out.printf("Solar time: %d:%02d%s\n", solarH, solarM, dayStr);

      double timeMin = getInputMin();
      int timeDay = (timeMin < 0) ? -1 : (timeMin >= 1440) ? 1 : 0;
      timeMin -= 1440 * timeDay;
      int timeH = (int) Math.floor(timeMin / 60);
      int timeM = (int) Math.floor(timeMin - 60.0d * timeH);
      String timeStr = (timeDay == -1) ? "-" : (timeDay == 1) ? "+" : "";
      System.out.printf("Clock time: %d:%02d%s\n", timeH, timeM, timeStr);

      double hourAngle = (solarMin - 720) / 4 * -1;
      System.out.printf("Hour angle: %.3f%s\n", hourAngle, degSign());

      double altAngle = getAltAngle(declAngle, latitude, solarHourAngle);
      System.out.printf("Altitude angle: %.3f%s\n", altAngle, degSign());

      double aziAngle = getPreAzimuth(declAngle, latitude, solarHourAngle,
       altAngle) - 180.0;
      aziAngle *= (hourAngle > 0 && hourAngle < 180) ? -1 : 1;
		System.out.printf("Azimuth angle: %.3f%s\n", aziAngle, degSign());

		//At this point, we have the compass heading.
		double trueNorth = norm360(180.0 - aziAngle); //true north
		double compHdg = trueNorth + mMagDecl; //magnetic north
		System.out.printf("*** Ideal compass headings are: ***\n");
		System.out.printf("From TRUE NORTH: %.3f%s\n", trueNorth, degSign());
		System.out.printf("From MAGNETIC NORTH: %.3f%s\n", compHdg, degSign());
      double daylightMin = getDaylightMin(declAngle, latitude, elev);
      double sunriseMin = 720 - daylightMin + 60 * deltaLSOT;
      int[] riseHrMin = getHourMin(sunriseMin);
      System.out.printf("Sunrise: %d:%02d\n", riseHrMin[0], riseHrMin[1]);
      double sunsetMin = 720 + daylightMin + 60 * deltaLSOT;
      int[] setHrMin = getHourMin(sunsetMin);
      System.out.printf("Sunset: %d:%02d\n", setHrMin[0], setHrMin[1]);
   }

   private void runFast()
   {
      float longitude = (float) mLong;
      float latitude = (float) mLat;
      float elev = (float) mElev;
      float offset = (float) mGMTOffset;

      float utslo = (float) getInputHr() - offset;
      float tVal = getT(utslo);
      float gVal = norm360(357.528f + 35999.05f * tVal);

      float cVal = getC(gVal);
      float lVal = norm360(280.46f + (tVal * 36000.77f) + cVal);

      float alpha = getAlpha(lVal);
      float ghaVal = norm360(utslo * 15 - 180 - cVal + lVal - alpha);

      float obliqAngle = 23.4393f - 0.013f * tVal;

      float declAngle = getDeclAngle(obliqAngle, alpha);

      System.out.printf("Declination: %.3f%s\n", declAngle, degSign());

      float deltaEOT = (lVal - cVal - alpha) / 15.0f; //EOT (hours)
      float totalSecs = deltaEOT * 3600; //EOT (total seconds)
      int minEOT = (int) Math.floor(totalSecs / 60.0f); //EOT (min)
      int secEOT = (int) Math.round(totalSecs - 60 * minEOT); //EOT (sec)

      System.out.printf("Equation of time (min): %d:%02d\n", minEOT, secEOT);

      //DST gets added to this if active
      float deltaLSOT = (float) (-longitude + (15 * offset)) / 15 - deltaEOT;

      float solarHourAngle = norm180(ghaVal + longitude); //GHA - signed long
      float solarTime = norm24(12 + solarHourAngle / 15.0f);

      float solarMin = (float) getInputMin() - deltaLSOT * 60.0f;
      int solarDay = (solarMin < 0) ? -1 : (solarMin >= 1440) ? 1 : 0;
      solarMin -= 1440 * solarDay;
      int solarH = (int) Math.floor(solarMin / 60);
      int solarM = (int) Math.floor(solarMin - 60.0d * solarH);
      String dayStr = (solarDay == -1) ? "-" : (solarDay == 1) ? "+" : "";
      System.out.printf("Solar time: %d:%02d%s\n", solarH, solarM, dayStr);

      float timeMin = (float) getInputMin();
      int timeDay = (timeMin < 0) ? -1 : (timeMin >= 1440) ? 1 : 0;
      timeMin -= 1440 * timeDay;
      int timeH = (int) Math.floor(timeMin / 60);
      int timeM = (int) Math.floor(timeMin - 60.0d * timeH);
      String timeStr = (timeDay == -1) ? "-" : (timeDay == 1) ? "+" : "";
      System.out.printf("Clock time: %d:%02d%s\n", timeH, timeM, timeStr);

      float hourAngle = (solarMin - 720) / 4 * -1;
      System.out.printf("Hour angle: %.3f%s\n", hourAngle, degSign());

      float altAngle = getAltAngle(declAngle, latitude, solarHourAngle);
      System.out.printf("Altitude angle: %.3f%s\n", altAngle, degSign());

      float aziAngle = getPreAzimuth(declAngle, latitude, solarHourAngle,
       altAngle) - 180.0f;
      aziAngle *= (hourAngle > 0 && hourAngle < 180) ? -1 : 1;
      System.out.printf("Azimuth angle: %.3f%s\n", aziAngle, degSign());

      float daylightMin = getDaylightMin(declAngle, latitude, elev);
      float sunriseMin = 720 - daylightMin + 60 * deltaLSOT;
      int[] riseHrMin = getHourMin(sunriseMin);
      System.out.printf("Sunrise: %d:%02d\n", riseHrMin[0], riseHrMin[1]);
      float sunsetMin = 720 + daylightMin + 60 * deltaLSOT;
      int[] setHrMin = getHourMin(sunsetMin);
      System.out.printf("Sunset: %d:%02d\n", setHrMin[0], setHrMin[1]);
   }

   public static void main(String[] args)
   {
      SunLocator sloc = new SunLocator();
      sloc.run();
      System.out.println("\nNow as optimized for the AVR microcontroller:\n");
      sloc.runFast();
   }
}
