
#include <stdlib.h>
#include <stdint.h>
#include <fstream>
#include <iostream>

#define F_CPU (16000000UL)
#define F_MHZ ((F_CPU)/1000000UL)

typedef union
{
	uint32_t whole;							///< All the data as one 32-bit number
	uint16_t half[2];						///< The data as an array of 16-bit ints
	uint8_t quarters[4];					///< The data as an array of 8-bit chars
} time_data_32;

typedef union
{
  uint64_t whole;							///< All the data as one 64-bit number
  uint32_t half[2];						///< The data as an array of 32-bit int
  uint16_t quarters[4];					///< The data as an array of 16-bit int
  uint8_t eighths[8];					///< The data as an array of 8-bit int
} time_data_64;

uint64_t getMicrosec(time_data_64 timeData)
{
  uint64_t data = (timeData.whole * 8ULL) / F_MHZ;
  return data;
}

uint32_t getSec(time_data_64 timeData)
{
  uint64_t usec = getMicrosec(timeData);
  return (uint32_t) (usec / 1000000ULL);
}

uint32_t getMin(time_data_64 timeData)
{
  uint32_t secs = getSec(timeData);
  return secs / 60UL;
}

uint32_t getHours(time_data_64 timeData)
{
  uint32_t mins = getMin(timeData);
  return mins / 60UL;
}

uint16_t getDays(time_data_64 timeData)
{
  uint32_t hours = getHours(timeData);
  return (uint16_t) (hours / 24UL);
}

