
/**
 * This class is intended to calibrate a motor.
 *
 * Calibrating a motor entails the following:
 *	1. Setting up 1 of 2 motor drivers
 *		a. Duty cycle starts out at 0 (control with Timer 1/Output 1A)
 *		b. The timeout period will be 250 milliseconds = 250e-3 sec.
 *	2. After setting the duty cycle and timeout period, 1 of 2 things will happen,
 *	   both of which will result in an interrupt taking place:
 *		a. Case 0: The timer reaches 250 milliseconds without any tick changes.
 *			1. There is not enough power going to the motor.
 *			2. Increase the power by an amount equal to DUTYCYCLE_STEP = 5%.
 *		b. Case 1: The optical encoder ticks before the 250ms is up.
 *			1. Case 1A: The motor is getting enough power to move.
 *				A. The minimum power is at some duty cycle between N and N - 5.
 *				B. Bring the motor back down to N - 10 (static friction takes back over)
 *				C. Increase the motor back up to N - 5
 *				D. Set DUTYCYCLE_STEP to 1 instead of 5.
 *				E. Repeat just like it was when N = 5 instead of 1.
 *				F. When the motor moves again, it should be between N - 5 and N, save this.
 *			2. Case 1B: The motor moved as a result of a force other than its own.
 *				A. This probably means the solar apparatus is outside and there are high winds.
 *				B. This also means the motor will need more power and be more time sensitive.
 *	3. After step 2, some semblance of a minimum duty cycle will have been determined. Next step
 *     is determining how far in a given direction the motor will move (which direction???)
 *		a1. Perhaps use push buttons to determine this?
 *		a2. Alternatively, such a directional limit is likely to result in a lower speed of
 *		    rotation for the optical encoders.
 *		b.  Whatever the underlying reason may be, if an abrupt change in speed occurs (out of
 *			proportion to changes of speed that may have occurred earlier), then:
 *			1. Halt motor movement at the present position and record as 0.
 *			2. Move motor in the other direction. The same phenomenon should occur (if this was
 *			   accurately determined) after about a thousand ticks in the other direction, which
 *			   translates to roughly 180 degrees. (Obviously MY_DEBUG(numTicks) is called for)
 *		c.	If the numbers work out to a sensible value, then its on the right track and passes the test.
 *
 *			