/** \file avr_macros.h
 *  This file contains a variety of commonly-used macros on the AVR.
 */

#ifndef _AVR_MACROS_H_
#define _AVR_MACROS_H_

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>

/**
 * Bit manipulation macros.
 */
#define BIT_POS(bit) (1 << (bit))
#define SET_0(reg,bit) ((reg) &= ~(BIT_POS(bit)))
#define SET_1(reg,bit) ((reg) |= (BIT_POS(bit)))
#define IS_1(reg,bit) (((reg) & BIT_POS(bit))?true:false)
#define IS_0(reg,bit) (!(IS_1((reg),(bit))))

#define PORT(portltr) (PORT##portltr)
#define PIN(portltr)  (PIN##portltr)
#define DDR(portltr)  (DDR##portltr)

/**
 * Shortcuts to common primitive types.
 */
#define U8 	  	uint8_t
#define S8		int8_t
#define U16		uint16_t
#define S16		int16_t
#define U32		uint32_t
#define S32		int32_t
#define U64		uint64_t
#define S64		int64_t
#define F32		float

/**
 * CPU frequency as F_CPU and F_MHZ.
 */
#ifndef F_CPU
  #define F_CPU (16000000UL)
#endif

#define F_MHZ ((F_CPU)/1000000UL)

#endif