
#ifndef _STL_TIMER2_H_
#define _STL_TIMER2_H_

#include <stdlib.h>							// Used for itoa()
#include <string.h>							// Header for character string functions
#include <avr/interrupt.h>					// For using interrupt service routines

#include "base_text_serial.h"				// Base for text-type serial port objects

// Check that the user has set the CPU frequency in the Makefile; if not, complain
#ifndef F_CPU
#error The macro F_CPU must be set in the Makefile.
#else
#define F_MHZ ((F_CPU)/1000000UL)
#endif

// If Timer 3 exists (as on the ATmega128), we use it for the microsecond timer rather
// than Timer 1, as Timer 1 is used for the motor PWM's on the ME405 board
#ifdef TCNT3
#define TMR_TCNT_REG		TCNT3					///< Register that holds the time count
#define TMR_intr_vect   TIMER3_OVF_vect	///< The timer overflow interrupt vector
//#define TMR_comp_vect	TIMER3_COMPA_vect ///< The timer compare A vector
//#define TMR_capt_vect	TIMER3_CAPT_vect  ///< The timer input capture vector
#else
#define TMR_TCNT_REG		TCNT1					///< Register that holds the time count
#define TMR_intr_vect   TIMER1_OVF_vect	///< The timer overflow interrupt vector
//#define TMR_comp_vect	TIMER1_COMPA_vect ///< The timer compare A vector
//#define TMR_capt_vect	TIMER1_CAPT_vect	///< The timer input capture vector
#endif // __AVR_ATmega128__

//--------------------------------------------------------------------------------------
/** This union holds a 32-bit time count. The count can be accessed as a single 32-bit
*  number, as two 16-bit integers placed together, or as an array of 8-bit characters. 
*  This is helpful because the time measurement we use consists of the number in a 
*  16-bit hardware counter and a 16-bit overflow count which should be put together 
*  into one time number. The characters are handy for looking at individual bits. 
*/
typedef union
{
  uint32_t whole;							///< All the data as one 32-bit number
  uint16_t half[2];						///< The data as an array of 16-bit ints
  uint8_t quarters[4];					///< The data as an array of 8-bit chars
} time_data_32;

typedef union
{
  uint64_t whole;
  uint32_t half[2];
  uint16_t quarters[4];
  uint8_t  eighths[8];
} time_data_64;

// 
#endif