/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author achernet
 */
public class SunLocator
{
    private double mLat = 35.3039605;
    private double mLat2 = 32.386677;
    private double mLong = -120.6635398;
    private double mLong2 = -111.043926;
    private double mGMTOffset = -8;
    private double mGMTOffset2 = -7;

    //default test time will be 1620 hours
    private double getInputHr()
    {
        return 16.0d + (20.0d / 60.0d);
    }

    private double getInputMin()
    {
        return 16.0d * 60.0d + 20.0d;
    }

    private int getInputYear() {
        return 2010;
    }

    private int getInputMonth() {
        return 2;
    }

    private int getInputDay() {
        return 26;
    }

    private int getInputDate()
    {
        int [] days = new int [] {
         0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
        return days[getInputMonth()-1] + getInputDay();
        //return 1265732400;
    }

    private int getCorrMonth() {
        int monthNo = getInputMonth();
        return (monthNo > 2) ? monthNo - 3 : monthNo + 9;
    }

    private int getCorrYear() {
        int yearNo = getInputYear();
        int monthNo = getInputMonth();
        return (monthNo > 2) ? yearNo : yearNo - 1;
    }

    private double getT(double univTime)
    {
        double univDays = univTime / 24.0d;
        int inputDate = getInputDate();
        int corrMonth = getCorrMonth();
        int corrYear = getCorrYear();
        double valMonth = Math.floor(30.6 * corrMonth + 0.5);
        double valYear = Math.floor(365.25 * (corrYear - 1976));
        double valT = univDays + inputDate + valMonth + valYear - 8707.5;
        return valT / 36525.0;
    }

    private double getC(double gVal)
    {
        double gRads = Math.toRadians(gVal);
        double valC1 = 1.915 * Math.sin(gRads);
        double valC2 = 0.020 * Math.sin(2.0 * gRads);
        return valC1 + valC2;
    }

    private double getAlpha(double lVal)
    {
        double lRads = Math.toRadians(lVal);
        double valAlpha1 = 2.466 * Math.sin(2.0 * lRads);
        double valAlpha2 = 0.053 * Math.sin(4.0 * lRads);
        return lVal - valAlpha1 + valAlpha2;
    }

    private double getDeclAngle(double obliqAngle, double alpha)
    {
        double obliqRad = Math.toRadians(obliqAngle);
        double alphaRad = Math.toRadians(alpha);
        double declRad = Math.atan(Math.tan(obliqRad) * Math.sin(alphaRad));
        return Math.toDegrees(declRad);
    }

    private double getAltAngle(double declAngle, double solarAngle)
    {
        double declRad = Math.toRadians(declAngle);
        double solarRad = Math.toRadians(solarAngle + 180);
        double latRad = Math.toRadians(mLat);
        double valAlt1 = Math.sin(latRad) * Math.sin(declRad);
        double valAlt2 = Math.cos(latRad) * Math.cos(declRad) *
         Math.cos(solarRad);
        return Math.toDegrees(Math.asin(valAlt1 - valAlt2));
    }

    private double norm360(double num) {
        return num - Math.floor(num / 360.0d) * 360;
    }

    private double norm180(double num) {
        double newNum = norm360(num);
        return (newNum > 180) ? newNum - 360 : newNum;
    }

    private double norm24(double num) {
        return num - Math.floor(num / 24.0) * 24;
    }

    private void run()
    {
        double utslo = getInputHr() - mGMTOffset;
        double tVal = getT(utslo);
        double gVal = norm360(357.528 + 35999.05 * tVal);

        double cVal = getC(gVal);
        double lVal = norm360(280.46 + (tVal * 36000.77) + cVal);

        double alpha = getAlpha(lVal);
        double ghaVal = norm360(utslo * 15 - 180 - cVal + lVal - alpha);

        double obliqAngle = 23.4393 - 0.013 * tVal;

        double declAngle = getDeclAngle(obliqAngle, alpha);
        System.out.printf("Declination: %.3f\n", declAngle);

        double deltaEOT = (lVal - cVal - alpha) / 15.0; //EOT (hours)
        System.out.printf("Equation of time: %.3f hours\n", deltaEOT);

        //DST gets added to this if active
        double deltaLSOT = mLong + (15 * mGMTOffset) / 15 - deltaEOT;

        double solarAngle = norm180(ghaVal + mLong); //
        double solarTime = norm24(12 + solarAngle / 15.0);

        double solarMin = getInputMin() - deltaLSOT * 60.0;
        int solarDay = (solarMin < 0) ? -1 : (solarMin >= 1440) ? 1 : 0;
        solarMin -= 1440 * solarDay;
        int solarH = (int) Math.floor(solarMin / 60);
        int solarM = (int) Math.floor(solarMin - 60.0d * solarH);
        String dayStr = (solarDay == -1) ? "-" : (solarDay == 1) ? "+" : "";
        System.out.printf("Solar time: %d:%d%s\n", solarH, solarM, dayStr);

        double timeMin = getInputMin();
        int timeDay = (timeMin < 0) ? -1 : (timeMin >= 1440) ? 1 : 0;
        timeMin -= 1440 * timeDay;
        int timeH = (int) Math.floor(timeMin / 60);
        int timeM = (int) Math.floor(timeMin - 60.0d * timeH);
        String timeStr = (timeDay == -1) ? "-" : (timeDay == 1) ? "+" : "";
        System.out.printf("Clock time: %d:%d%s\n", timeH, timeM, timeStr);

        double hourAngle = (solarMin - 720) / 4 * -1;
        System.out.printf("Hour angle: %.3f degrees\n", hourAngle);

        double altAngle = getAltAngle(declAngle, solarAngle);
        System.out.printf("Altitude angle: %.3f degrees\n", altAngle);


    }

    public static void main(String [] args)
    {
        SunLocator sloc = new SunLocator();
        sloc.run();
    }
}
