/**
 * \file lab4sim.cpp
 *
 * Simulates interface for duty cycle control for mechatronics.
 *
 *	Revisions:
 *		\li 01-30-10 Final version
 *		\li 02-01-10 Fixed the brake problem, fixed the ccw display problem
 *
 *	License:
 *		This file released under the LGPL.
 */
#include <stdlib.h>							// Standard C library
#include <avr/io.h>							// standard C++ input/ output library

#include "rs232int.h"						// serial port class
// #include "VNH3SP30_motor1.h"
// #include "VNH3SP30_motor2.h"
#include "VNH3SP30WithBitsHeader.h"

/**
 * Mode for testing program.
 */
uint8_t mode = 0;

/**
 * Stored duty cycle.
 */
int8_t dutyCycle = 0;

/**
 * Used to store a new duty cycle input.
 */
uint8_t newValue;

/**
 * Stored brake setting.
 */
bool brake;

/** 
 * Running brake count total.
 */
uint16_t brakecount;

/**
 * Maximum brake count before reset.
 */
const uint32_t brakevalue = 25;

/**
 * Duty cycle to set on brake.
 */
const uint8_t brakeduty = 100;

/**
 * Main function for Lab 3 -- the motor driver testing function.
 * @return zero
 */

/**
 * Stores the motor number currently being used by the user
 */
uint8_t channel = 0;

/**
 * Stores the direction of the current motor
 */
bool direction = true;

int main()
{
	mode = 0;
	dutyCycle = 0;
	//isCCW = true;

	sei();
	rs232 the_serial_port (9600, 1);

// 	VNH3SP30_motor1 my_motor1;
// 	VNH3SP30_motor2 my_motor2;
	VNH3SP30WithBitsHeader my_motor3;

	the_serial_port <<  "Mechatronics motor simulator v0.1!\n Press 'D' for directions \n Select motor \n";
	while(true)		// Loop always
	{
		while(true)		// Wait until a motor is selected
		{
			if (the_serial_port.check_for_char())	// Check if key is available
			{
				char chan = the_serial_port.getchar();  	// Fetch the key
				if (chan == '1')
				{	
					the_serial_port << "Using motor 1 \n";
					channel = 1;
					break;
				}
				else if (chan == '2')
				{
					the_serial_port << "Using motor 2 \n";
					channel = 2;
					break;
				}
				else
				{
					the_serial_port << "Incorrect entry; Please select correct motor \n";
				}
			}
		}
		while (true)
		{
			if (the_serial_port.check_for_char ()) //if (hasChar(serial)) ..
			{
				char ch = the_serial_port.getchar();
				if (ch == ' ' && mode == 0) //Request for new value with space key
				{
					the_serial_port << endl << "Got request for new value!\n" 
					<< endl << "Enter the duty cycle: " << endl;
					mode = 1;
				}
				else if (ch == ' ' && mode != 0) // End current operation with space key
				{
					the_serial_port << endl << "Aborting operation...\n" << endl;
					mode = 0;
				}
				else if (ch >= 0x30 && ch <= 0x39 && (mode == 1 || mode == 2)) // Intepret keypad entry 0-9 
				{
					uint8_t digit = ch - 0x30;
					newValue = (mode == 1) ? digit * 10 : newValue + digit;
					mode++;
				}
				else if (mode == 3 && (ch == '-' || ch == '+')) 
				{
	// 				direction = (ch == '-') ? /*TORQUE_CW */: /*TORQUE_CCW*/;  
					if (ch == '+')
					{
						direction = true;
						dutyCycle = newValue;
					}
					else if (ch == '-')
					{
						direction = false;
						dutyCycle = -newValue;
					}
					newValue = 0;
					mode = 0;
					
					/*if(channel == 1)
					{
						my_motor1.set_dutycycle(dutyCycle);
					}
					else if (channel == 2)
					{
						my_motor2.set_dutycycle(dutyCycle);
					}*/
					my_motor3.set_dutycycle(dutyCycle);
					the_serial_port << "\nNew motor setting read correctly!\n";
					the_serial_port << "Duty cycle: " << dutyCycle << "%" << endl;
					the_serial_port << "Direction: " << ((direction) ? "CLOCKWISE" : "COUNTERCLOCKWISE") << endl;
				}
				else if (mode == 0 && ((ch == 'b')|(ch == 'B'))&& (brakecount == 0))   //added braking here!!!!!!!!!!!!
				{
					brake = true;
					my_motor3.set_brake(brake);
					
// 					if(channel == 1)
// 					{
// 						my_motor1.set_brake(brake);
// 					}
// 					else if (channel == 2)
// 					{
// 						my_motor2.set_brake(brake);
// 					}
					the_serial_port << endl << "\nERRRRRRRRRRRRRRRRHHHHHH\n" << endl;
				}			
				else if ((brakecount <= brakevalue) && (brakecount != 0))
				{
					++brakecount;
				}
				else if (brakecount>= brakevalue)
				{
					brakecount = 0;
					brake = false;
					my_motor3.set_brake(brake);
					my_motor3.set_dutycycle(dutyCycle);
// 					if (channel == 1)
// 					{
// 						my_motor1.set_brake(brake);
// 						my_motor1.set_dutycycle(dutyCycle);
// 					}
// 					else if (channel == 2)
// 					{
// 						my_motor2.set_brake(brake);
// 						my_motor2.set_dutycycle(dutyCycle);
// 					}
				}
				else if (ch == 'c' || ch == 'C')
				{
					the_serial_port << endl << "Quitting current motor\n" << endl;
					break;
				}
				else if (ch == 'p' || ch == 'P')
				{
					the_serial_port << endl << "Current settings for motor:\n" << endl << "Duty cycle: %d%%\n" 
					<< endl << dutyCycle << endl << "Direction: " << endl << direction << endl; 
				}
				else if (ch == 'd' || ch == 'D')
				{
					the_serial_port << "Directions: \n \
						1) Press 1 or 2 for the desired motor \n \
						2) Press space to request new 2 digit value \n \
						3) Type in 2 digit value \n \
						4) Press '-' or '+' keys for direction \n \
						5) Press 'b' to apply a brake \n \
						**Press spacebar at anytime to abort operaion \n \
						**Press 'c' to change current motor" << endl;
				}
			}
		}
	}
	the_serial_port.getchar();
	return 0;
}



	/*cout << "Mechatronics motor simulator v0.1\n";
	cout << "This should be an easy interface. To key in a new duty\n";
	cout << "cycle, just hit the spacebar, type in 2 numbers 00 to 99,\n";
	cout << "and then hit + or - for counterclockwise or clockwise motion\n";
	cout << "Hit spacebar again during the prompt to cancel it. Otherwise,\n";
	cout << "when not in prompt mode, hit \'b\' to brake motor to GND and\n";
	cout << "\'B\' to brake motor to VCC. Ready?\n\n";
	
	while (true)
	{

}*/
