
#include <curses.h>
#include <stdlib.h>
#include <fstream>
#include <iostream>

using namespace std;

//coordinates for the lab
float mLat = 35.3039605;
float mLong = -120.6635398;

uint8_t mHour = 0;
uint8_t mMin = 0;
int8_t mGMTOffset = -8;

void showMainMenu()
{
	cout << "Choose an option:\n\n";
	cout << "\t- t: set the time\n";
	cout << "\t- d: set the date\n";
	cout << "\t- c: set the coordinates\n";
	cout << "\t- z: set the time zone\n";
	cout << "\t- p: print current settings\n";
	cout << "\t- s: compute solar calculations\n";
	cout << "\t- h: help\n";
	cout << "\t- q: quit screen\n";
}

int8_t enterTimezone()
{
	int8_t timeEntered = '\0';
	while (true)
	{
		cout << "Enter your timezone:\n\n";
		cout << "\t- P: Pacific Time (GMT - 8)\n";
		cout << "\t- M: Mountain Time (GMT - 7)\n";
		cout << "\t- C: Central Time (GMT - 6)\n";
		cout << "\t- E: Eastern Time (GMT - 5)\n";
		cout << "Or enter a number from -12 to 12: ";
		cin >> timeEntered;
		switch (timeEntered)
		{
			case 'P': case 'p':
				return -8;
			case 'M': case 'm':
				return -7;
			case 'C': case 'c':
				return -6;
			case 'E': case 'e':
				return -5;
			default:
				if (timeEntered >= -12 && timeEntered <= 12) 
				{ 
					return timeEntered;
				}
				cout << "Not a recognized time zone\n";
				break;
		}
	}
	return timeEntered;
}

uint8_t enterHour()
{
	uint8_t hour = 255;
	int8_t ampm = '\0';
	string hour_str;
	while (true)
	{
		cout << "Enter the hour of day:\n\n";
		cin >> hour;
		
		if (hour >= 24)
		{
			cout << "Not a valid hour of day (> 24)\n";
			continue;
		}
		if (hour > 12 || hour == 0)
		{
			return hour;
		}
		else
		{
			cout << "Enter A for AM or P for PM (default: AM)\n";
			cin >> ampm;
			if (ampm == 'p' || ampm == 'P')
			{
				hour += (hour == 12) ? 0 : 12;
			}
			else
			{
				hour = (hour == 12) ? 0 : hour;
			}
			return hour;
		}
	}
	return hour;
}

uint8_t enterMin()
{
	uint8_t min = 255;
	while (true)
	{
		cout << "Enter the minute:\n\n";
		cin >> min;
		if (min > 59)
		{
			cout << "Not a valid minute (> 59)\n";
			continue;
		}
		return min;
	}
}

void enterTime()
{
	cout << "This will set the current time of day.\n";
	mGMTOffset = enterTimezone();
	mHour = enterHour();
	mMin = enterMin();
}

uint8_t getAnOption()
{
	uint8_t ch = '\0';
	while (true)
	{
		cin >> ch;
		switch (ch)
		{
			case 't': case 'T':
				return 't';
			case 'c': case 'C':
				return 'c';
			case 'z': case 'Z':
				return 'z';
			case 'p': case 'P':
				return 'p';
			case 's': case 'S':
				return 's';
			case 'h': case 'H':
				return 'h';
			case 'q': case 'Q':
				return 'q';
			default: break;
		}
	}
	return ch;
}

void printSettings()
{
	cout << "Latitude: " << mLat << " Longitude: " << mLong << "\n";
	cout << "Time: " << mHour << ":" << mMin;
}

int main()
{
	cout << "Menu testing\n";
	showMainMenu();
	while (true)
	{
		uint8_t option = getAnOption();
		if (option == 't')
		{
			enterTime();
		}
		if (option == 'p')
		{
			printSettings();
		}
		if (option == 'q')
		{
			continue;
		}
		if (option == 'h')
		{
			showMainMenu();
		}
	}
	return 0;
}




